from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_registration.models import RegisteredSubject

from ..admin_site import meta_subject_admin
from ..forms import BirthOutcomesForm
from ..models import BirthOutcomes


@admin.register(BirthOutcomes, site=meta_subject_admin)
class BirthOutcomesAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = BirthOutcomesForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Birth Outcome",
            {
                "fields": (
                    "birth_order",
                    "birth_outcome",
                    "birth_weight",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_visit",
        "delivery_report",
        "dashboard",
        "birth_order",
        "birth_outcome",
    )

    list_filter = (
        "birth_order",
        "birth_outcome",
    )

    radio_fields = {
        "birth_outcome": admin.VERTICAL,
    }

    search_fields = (
        "subject_visit__subject_identifier",
        "delivery__action_identifier",
        "delivery__tracking_identifier",
    )

    @admin.display
    def delivery_report(self, obj=None, label=None):
        url = reverse("meta_prn_admin:meta_prn_delivery_changelist")
        url = f"{url}?q={obj.subject_identifier}"
        context = dict(title="Delivery", url=url, label="Delivery")
        return render_to_string("dashboard_button.html", context=context)

    def get_subject_dashboard_url_kwargs(self, obj):
        return dict(subject_identifier=obj.subject_identifier)

    def view_on_site(self, obj):
        try:
            RegisteredSubject.objects.get(subject_identifier=obj.subject_identifier)
        except ObjectDoesNotExist:
            url = reverse(self.get_subject_listboard_url_name())
        else:
            try:
                url = reverse(
                    self.get_subject_dashboard_url_name(),
                    kwargs=self.get_subject_dashboard_url_kwargs(obj),
                )
            except NoReverseMatch as e:
                if callable(super().view_on_site):
                    url = super().view_on_site(obj)
                else:
                    raise NoReverseMatch(
                        f"{e}. See subject_dashboard_url_name for {repr(self)}."
                    )
        return url



from django.db import models
from django.db.models import PROTECT
from edc_crf.crf_with_action_model_mixin import CrfWithActionModelMixin
from edc_model import models as edc_models
from edc_utils import get_utcnow

from ..choices import FETAL_OUTCOMES
from .delivery import Delivery


class BirthOutcomes(
    CrfWithActionModelMixin,
    edc_models.BaseUuidModel,
):

    delivery = models.ForeignKey(Delivery, on_delete=PROTECT)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
    )

    birth_order = models.IntegerField()

    birth_outcome = models.CharField(
        verbose_name="Outcome", max_length=25, choices=FETAL_OUTCOMES
    )

    birth_weight = models.IntegerField(
        verbose_name="Weight (gm)", help_text="gm", null=True, blank=True
    )

    def __str__(self):
        return f"{self.subject_identifier} #{self.birth_order or '-'}"

    def save(self, *args, **kwargs):
        self.report_datetime = self.delivery.report_datetime
        self.subject_visit = self.delivery.subject_visit
        super().save(*args, **kwargs)

    class Meta(edc_models.BaseUuidModel.Meta):
        verbose_name = "Birth Outcomes"
        verbose_name_plural = "Birth Outcomes"
        ordering = ["delivery", "birth_order"]
        unique_together = ["delivery", "birth_order"]



from edc_constants.constants import NO, NOT_APPLICABLE, PENDING, TBD, YES
from edc_screening.screening_eligibility import FC, ScreeningEligibilityError

from ...constants import EGFR_LT_45, FBG_OGTT_INCOMPLETE, IFT_OGTT, SEVERE_HTN
from .base_eligibility_part_three import BaseEligibilityPartThree


class EligibilityPartThreePhaseThree(BaseEligibilityPartThree):
    def __init__(self, **kwargs):
        self.inclusion_a = TBD
        self.inclusion_b = TBD
        self.severe_htn = None
        super().__init__(**kwargs)

    def assess_eligibility(self) -> None:
        """Subject is eligible if either a qualifying FBG OR a qualifying OGTT"""
        # TODO: check if calculates correctly if glucose is HIGH 9999.99
        super().assess_eligibility()
        self.calculate_inclusion_field_values()
        if self.inclusion_a == TBD or self.inclusion_b == TBD:
            self.reasons_ineligible.update(fbg_ogtt=FBG_OGTT_INCOMPLETE)
            self.eligible = TBD
        elif self.inclusion_a == NO and self.inclusion_b == NO:
            self.reasons_ineligible.update(fbg_ogtt=IFT_OGTT)
            self.eligible = NO
        elif self.calculated_egfr_value and self.calculated_egfr_value < 45.0:
            self.reasons_ineligible.update(egfr=EGFR_LT_45)
            self.eligible = NO

    def set_fld_attrs_on_model(self) -> None:
        """Set extra attr on the model"""
        super().set_fld_attrs_on_model()
        self.model_obj.inclusion_a = self.inclusion_a
        self.model_obj.inclusion_b = self.inclusion_b
        self.model_obj.inclusion_c = NOT_APPLICABLE
        self.model_obj.inclusion_d = NOT_APPLICABLE

    def get_required_fields(self) -> dict[str, FC]:
        """Get default fields as well as severe_htn"""
        fields = super().get_required_fields()
        fields.update({"severe_htn": FC(NO, SEVERE_HTN)})
        return fields

    def calculate_inclusion_field_values(self) -> None:
        """Decide which values to use, first or repeat"""
        if self.repeat_glucose_performed == PENDING:
            self.inclusion_a = TBD
            self.inclusion_b = TBD
        else:
            converted_fbg_value = (
                self.converted_fbg2_value
                if self.repeat_glucose_opinion == YES
                else self.converted_fbg_value
            )
            converted_ogtt_value = (
                self.converted_ogtt2_value
                if self.repeat_glucose_opinion == YES
                else self.converted_ogtt_value
            )
            # FBG (6.1 to 6.9 mmol/L)
            # TODO ensure only 1 decimal place is reported on the form
            if not converted_fbg_value:
                self.inclusion_a = TBD
            elif 6.1 <= converted_fbg_value < 7.0:
                self.inclusion_a = YES
            else:
                self.inclusion_a = NO

            # OGTT (7.8 to 11.10 mmol/L)
            if not converted_ogtt_value:
                self.inclusion_b = TBD
            elif 7.8 <= converted_ogtt_value < 11.1:
                self.inclusion_b = YES
            else:
                self.inclusion_b = NO



from edc_list_data.model_mixins import ListModelMixin


class AbnormalFootAppearanceObservations(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Abnormal Foot Appearance Observations"
        verbose_name_plural = "Abnormal Foot Appearance Observations"


class Symptoms(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Symptoms"
        verbose_name_plural = "Symptoms"


class ArvRegimens(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "ARV Regimens"
        verbose_name_plural = "ARV Regimens"


class OffstudyReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Offstudy Reasons"
        verbose_name_plural = "Offstudy Reasons"


class OiProphylaxis(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "OI Prophylaxis"
        verbose_name_plural = "OI Prophylaxis"


class BaselineSymptoms(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Baseline Symptoms"
        verbose_name_plural = "Baseline Symptoms"


class DiabetesSymptoms(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Diabetes Symptoms"
        verbose_name_plural = "Diabetes Symptoms"


class HypertensionMedications(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Hypertension Medications"
        verbose_name_plural = "Hypertension Medications"


class NonAdherenceReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Non-Adherence Reasons"
        verbose_name_plural = "Non-Adherence Reasons"


class SubjectVisitMissedReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Subject Missed Visit Reasons"
        verbose_name_plural = "Subject Missed Visit Reasons"



from django import forms
from edc_adverse_event.form_validator_mixins import (
    RequiresDeathReportFormValidatorMixin,
)
from edc_consent.constants import CONSENT_WITHDRAWAL
from edc_constants.constants import DEAD, OTHER
from edc_form_validators import FormValidator
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_ltfu.modelform_mixins import RequiresLtfuFormValidatorMixin

from ..constants import OTHER_RX_DISCONTINUATION


class EndOfStudyPhaseTwoFormValidator(
    RequiresDeathReportFormValidatorMixin,
    RequiresLtfuFormValidatorMixin,
    FormValidator,
):
    death_report_model = "meta_ae.deathreport"
    ltfu_model = None

    def clean(self):

        self.validate_death_report_if_deceased()
        self.validate_ltfu()

        if self.cleaned_data.get("offschedule_reason"):
            if self.cleaned_data.get("offschedule_reason").name != OTHER:
                self.validate_other_specify(
                    field="offschedule_reason",
                    other_specify_field="other_offschedule_reason",
                    other_stored_value=OTHER_RX_DISCONTINUATION,
                )

            if (
                self.cleaned_data.get("offschedule_reason").name
                != OTHER_RX_DISCONTINUATION
            ):
                self.validate_other_specify(
                    field="offschedule_reason",
                    other_specify_field="other_offschedule_reason",
                    other_stored_value=OTHER,
                )

        self.required_if(DEAD, field="offschedule_reason", field_required="death_date")

        self.required_if(
            LOST_TO_FOLLOWUP,
            field="offschedule_reason",
            field_required="ltfu_date",
        )

        self.required_if(
            CONSENT_WITHDRAWAL,
            field="offschedule_reason",
            field_required="consent_withdrawal_reason",
        )

        self.required_if(
            "included_in_error",
            field="offschedule_reason",
            field_required="included_in_error",
        )

        self.required_if(
            "included_in_error",
            field="offschedule_reason",
            field_required="included_in_error_date",
        )


class EndOfStudyPhaseThreeFormValidator(
    RequiresDeathReportFormValidatorMixin,
    RequiresLtfuFormValidatorMixin,
    FormValidator,
):
    death_report_model = "meta_ae.deathreport"
    ltfu_model = None

    def clean(self):

        if self.cleaned_data.get("offschedule_datetime") and self.cleaned_data.get(
            "last_seen_date"
        ):
            if (
                self.cleaned_data.get("last_seen_date")
                > self.cleaned_data.get("offschedule_datetime").date()
            ):
                raise forms.ValidationError(
                    {"last_seen_date": "Invalid. May not be after termination date"}
                )

        self.validate_death_report_if_deceased()
        self.validate_ltfu()

        if self.cleaned_data.get("offschedule_reason"):
            if self.cleaned_data.get("offschedule_reason").name != OTHER:
                self.validate_other_specify(
                    field="offschedule_reason",
                    other_specify_field="other_offschedule_reason",
                    other_stored_value=OTHER_RX_DISCONTINUATION,
                )

            if (
                self.cleaned_data.get("offschedule_reason").name
                != OTHER_RX_DISCONTINUATION
            ):
                self.validate_other_specify(
                    field="offschedule_reason",
                    other_specify_field="other_offschedule_reason",
                    other_stored_value=OTHER,
                )

        self.required_if(DEAD, field="offschedule_reason", field_required="death_date")

        self.required_if(
            LOST_TO_FOLLOWUP,
            field="offschedule_reason",
            field_required="ltfu_date",
        )

        self.required_if(
            CONSENT_WITHDRAWAL,
            field="offschedule_reason",
            field_required="consent_withdrawal_reason",
        )

        self.required_if(
            "included_in_error",
            field="offschedule_reason",
            field_required="included_in_error",
        )

        self.required_if(
            "included_in_error",
            field="offschedule_reason",
            field_required="included_in_error_date",
        )



# Generated by Django 3.2.11 on 2022-01-28 14:19

from django.db import migrations, models
import edc_model.models.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0015_auto_20211104_1447'),
    ]

    operations = [
        migrations.AddField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('pregnancy', 'Pregnancy'), ('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn for `clinical` reasons, please specify ...'),
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name='If other `clinical` reason, please specify ...'),
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='last_seen_date',
            field=models.DateField(null=True, validators=[edc_model.models.validators.date.date_not_future], verbose_name='Date patient was last seen'),
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='toxicity_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('lactic_acidosis', 'Development of lactic acidosis or hyperlactatemia'), ('hepatomegaly', 'Development of hepatomegaly with steatosis'), ('OTHER', 'Other (specify below)')], max_length=25, null=True, verbose_name='If withdrawn for a `toxicity`, please specify ...'),
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='toxicity_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name='If other `toxicity`, please specify ...'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('pregnancy', 'Pregnancy'), ('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn for `clinical` reasons, please specify ...'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name='If other `clinical` reason, please specify ...'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='last_seen_date',
            field=models.DateField(null=True, validators=[edc_model.models.validators.date.date_not_future], verbose_name='Date patient was last seen'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='toxicity_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('lactic_acidosis', 'Development of lactic acidosis or hyperlactatemia'), ('hepatomegaly', 'Development of hepatomegaly with steatosis'), ('OTHER', 'Other (specify below)')], max_length=25, null=True, verbose_name='If withdrawn for a `toxicity`, please specify ...'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='toxicity_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name='If other `toxicity`, please specify ...'),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models

from ..choices import (
    ACTIVITY_CHOICES,
    CHILDCARE_CHOICES,
    PAYEE_CHOICES,
    TRANSPORT_CHOICES,
)
from ..model_mixins import CrfModelMixin


class HealthEconomics(CrfModelMixin, edc_models.BaseUuidModel):

    occupation = models.CharField(
        verbose_name="What is your occupation/profession?", max_length=50
    )

    education_in_years = models.IntegerField(
        verbose_name="How many years of education did you compete?",
        validators=[MinValueValidator(0), MaxValueValidator(30)],
    )

    education_certificate = models.CharField(
        verbose_name="What is your highest education certificate?", max_length=50
    )

    primary_school = models.CharField(
        verbose_name="Did you go to primary/elementary school?",
        max_length=15,
        choices=YES_NO,
    )

    primary_school_in_years = models.IntegerField(
        verbose_name="If YES, for how many years",
        validators=[MinValueValidator(0), MaxValueValidator(10)],
    )

    secondary_school = models.CharField(
        verbose_name="Did you go to secondary school?", max_length=15, choices=YES_NO
    )

    secondary_school_in_years = models.IntegerField(
        verbose_name="If YES, for how many years",
        validators=[MinValueValidator(0), MaxValueValidator(10)],
        null=True,
        blank=True,
    )

    higher_education = models.CharField(
        verbose_name="Did you go to higher education?", max_length=15, choices=YES_NO
    )

    higher_education_in_years = models.IntegerField(
        verbose_name="If YES, for how many years",
        validators=[MinValueValidator(0), MaxValueValidator(10)],
        null=True,
        blank=True,
    )

    welfare = models.CharField(
        verbose_name="Do you receive any welfare or social service support",
        max_length=15,
        choices=YES_NO,
    )

    income_per_month = models.IntegerField(
        verbose_name="How much do you earn (take home) per month?",
        help_text="in Shilling",
    )

    household_income_per_month = models.IntegerField(
        verbose_name="What is the total income in your household per month?",
        help_text="in Shilling",
    )

    is_highest_earner = models.CharField(
        verbose_name=(
            "Are you the person who earns the highest income in your household?"
        ),
        max_length=15,
        choices=YES_NO,
    )

    highest_earner = models.CharField(
        verbose_name=(
            "If NO, what is the profession of the person "
            "who earns the highest income?"
        ),
        max_length=50,
        null=True,
        blank=True,
    )

    food_per_month = models.IntegerField(
        verbose_name="How much do you/your family spend on food in a month?",
        help_text="in Shilling",
    )

    accomodation_per_month = models.IntegerField(
        verbose_name=(
            "How much do you/your family spend on rent and utilities in a month?"
        ),
        help_text="in Shilling",
    )

    large_expenditure_year = models.IntegerField(
        verbose_name="How much have you spent on large items in the last year",
        help_text="e.g. furniture, electrical items, cars (in Shilling)",
    )

    buy_meds_month = models.CharField(
        verbose_name="Over the last month, did you get any drugs or have a drug refill?",
        max_length=15,
        choices=YES_NO,
    )

    # removed
    diabetic_expenditure_month = models.IntegerField(
        verbose_name="If YES, how much did you spend on Diabetic drugs",
        help_text="in Shilling",
        null=True,
        blank=True,
        editable=False,
    )

    # removed
    diabetic_payee = models.CharField(
        verbose_name="If YES, how did you pay or who paid for these drugs",
        max_length=15,
        choices=PAYEE_CHOICES,
        editable=False,
    )

    hypertensive_expenditure_month = models.IntegerField(
        verbose_name="If YES, how much did you spend on Hypertensive drugs",
        help_text="in Shilling",
        null=True,
        blank=True,
    )

    hypertensive_payee = models.CharField(
        verbose_name="If YES, how did you pay or who paid for these drugs",
        max_length=15,
        choices=PAYEE_CHOICES,
    )

    arv_expenditure_month = models.IntegerField(
        verbose_name="If YES, how much did you spend on Antiretroviral drugs",
        help_text="in Shilling",
        null=True,
        blank=True,
    )

    arv_payee = models.CharField(
        verbose_name="If YES, how did you pay or who paid for these drugs",
        max_length=15,
        choices=PAYEE_CHOICES,
    )

    meds_other_expenditure_month = models.IntegerField(
        verbose_name="If YES, how much did you spend on 'other' drugs",
        help_text="in Shilling",
        null=True,
        blank=True,
    )

    meds_other_payee = models.CharField(
        verbose_name="If YES, how did you pay or who paid for these drugs",
        max_length=15,
        choices=PAYEE_CHOICES,
    )

    expenditure_other_month = models.CharField(
        verbose_name=(
            "Over the last month, did you spend money on other activities (not drugs) "
            "relating to your health?"
        ),
        max_length=15,
        choices=YES_NO,
    )

    expenditure_other_detail = models.TextField(
        verbose_name="If YES, what was the activity", null=True, blank=True
    )

    expenditure_other = models.IntegerField(
        verbose_name=(
            "If YES, how much was spent on other activities "
            "(not drugs) relating to your health?"
        ),
        help_text="in Shilling",
        null=True,
        blank=True,
    )

    expenditure_other_payee = models.CharField(
        verbose_name="If YES, how did you pay or who paid for these activities?",
        max_length=15,
        choices=PAYEE_CHOICES,
    )

    healthcare_expenditure_month = models.IntegerField(
        verbose_name=(
            "How much in total has been spent on your healthcare in the last month?"
        )
    )

    routine_activities = models.CharField(
        verbose_name="What would you be doing if you had not come to the hospital?",
        max_length=25,
        choices=ACTIVITY_CHOICES,
    )

    routine_activities_other = models.CharField(
        verbose_name="If OTHER, please specify", max_length=50, null=True, blank=True
    )

    off_work_days = models.DecimalField(
        verbose_name="How much time did you take off work?",
        decimal_places=1,
        max_digits=4,
        help_text="in days. (1,2,3 etc. If half-day 0.5)",
    )

    travel_time = models.CharField(
        verbose_name="How long did it take you to reach here?",
        max_length=5,
        help_text="in hours and minutes (format HH:MM)",
    )

    hospital_time = models.CharField(
        verbose_name="How much time did you spend at the hospital?",
        max_length=5,
        help_text="in hours and minutes (format HH:MM)",
    )

    lost_income = models.CharField(
        verbose_name="Did you lose earnings as a result? ",
        max_length=15,
        choices=YES_NO,
    )

    lost_income_amount = models.IntegerField(
        verbose_name="If Yes, how much did you lose?", null=True, blank=True
    )

    childcare = models.CharField(
        verbose_name=(
            "Did you ask anyone else, such as your family member, "
            "friend to look after your child/children in order to come here?"
        ),
        max_length=15,
        choices=YES_NO_NA,
    )

    childcare_source = models.CharField(
        verbose_name=(
            "If Yes, what would they have been doing if they had not stayed to "
            "look after your child or children?"
        ),
        max_length=25,
        choices=CHILDCARE_CHOICES,
        default=NOT_APPLICABLE,
    )

    childcare_source_timeoff = models.DecimalField(
        verbose_name=(
            "How much time did a family member, friend take off work to look "
            "after your child or children?"
        ),
        decimal_places=1,
        max_digits=4,
        null=True,
        blank=True,
        help_text="in days. (1,2,3 etc. If half-day 0.5)",
    )

    # change to M2M, add field for other sepcify.
    transport_old = models.CharField(
        verbose_name=(
            "Which form of transport did you take to get to the hospital today?"
        ),
        max_length=25,
        choices=TRANSPORT_CHOICES,
    )

    transport_cost = models.IntegerField(
        verbose_name="How much did you spend on transport in total?",
        help_text="in Shilling",
    )

    transport_barter = models.CharField(
        verbose_name="Did you sell anything to pay for your visit today?",
        max_length=15,
        choices=YES_NO,
    )

    transport_borrow = models.CharField(
        verbose_name="Did you take any loans to pay for your visit?",
        max_length=15,
        choices=YES_NO,
    )

    # 39
    health_insurance = models.CharField(
        verbose_name="Do you have private healthcare insurance?",
        max_length=15,
        choices=YES_NO,
    )

    health_insurance_pay = models.IntegerField(
        verbose_name=(
            "If Yes, how much do you pay towards your contributions to "
            "healthcare insurance every month?"
        ),
        null=True,
        blank=True,
        help_text="in Shilling",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Health Economics (M2)"
        verbose_name_plural = "Health Economics (M2)"



# Generated by Django 3.2.11 on 2022-04-07 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0042_auto_20220403_1402'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='As reported by patient', max_length=15, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('PENDING', 'Pending (scheduled for 3 days from first)'), ('Yes', 'Yes'), ('N/A', 'Not applicable')], default='N/A', help_text='Select YES when you are ready to enter the repeat results', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='As reported by patient', max_length=15, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('PENDING', 'Pending (scheduled for 3 days from first)'), ('Yes', 'Yes'), ('N/A', 'Not applicable')], default='N/A', help_text='Select YES when you are ready to enter the repeat results', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='As reported by patient', max_length=15, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('PENDING', 'Pending (scheduled for 3 days from first)'), ('Yes', 'Yes'), ('N/A', 'Not applicable')], default='N/A', help_text='Select YES when you are ready to enter the repeat results', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='As reported by patient', max_length=15, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('PENDING', 'Pending (scheduled for 3 days from first)'), ('Yes', 'Yes'), ('N/A', 'Not applicable')], default='N/A', help_text='Select YES when you are ready to enter the repeat results', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='As reported by patient', max_length=15, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('PENDING', 'Pending (scheduled for 3 days from first)'), ('Yes', 'Yes'), ('N/A', 'Not applicable')], default='N/A', help_text='Select YES when you are ready to enter the repeat results', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
    ]



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_blood_results.form_validator_mixins import (
    BloodResultsFormValidatorMixin,
    BloodResultsGluFormValidatorMixin,
)
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import FormValidator
from edc_lab_panel.panels import blood_glucose_panel

from ...models import BloodResultsGlu


class BloodResultsGluFormValidator(
    BloodResultsGluFormValidatorMixin, BloodResultsFormValidatorMixin, FormValidator
):
    panel = blood_glucose_panel


class BloodResultsGluForm(ActionItemFormMixin, CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = BloodResultsGluFormValidator

    class Meta:
        model = BloodResultsGlu
        fields = "__all__"



from edc_crf.crf_model_mixin import CrfModelMixin as BaseCrfModelMixin
from edc_crf.crf_status_model_mixin import CrfStatusModelMixin
from edc_crf.crf_with_action_model_mixin import (
    CrfWithActionModelMixin as BaseCrfWithActionModelMixin,
)


class CrfModelMixin(CrfStatusModelMixin, BaseCrfModelMixin):
    class Meta(BaseCrfModelMixin.Meta):
        abstract = True


class CrfWithActionModelMixin(CrfStatusModelMixin, BaseCrfWithActionModelMixin):
    class Meta(BaseCrfModelMixin.Meta):
        abstract = True



from edc_model import models as edc_models
from edc_qol.model_mixins import Sf12ModelMixin

from ..model_mixins import CrfModelMixin


class Sf12(Sf12ModelMixin, CrfModelMixin, edc_models.BaseUuidModel):
    class Meta(Sf12ModelMixin.Meta, CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        pass


# TODO: add proxy model and two options Swahili and English



# Generated by Django 3.0.6 on 2020-06-13 17:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0008_auto_20200528_1517"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="MissedVisitReasons", new_name="SubjectVisitMissedReasons"
        )
    ]



DAY1 = "1000"
DELIVERY = "2000"
MONTH1 = "1010"
MONTH12 = "1120"
MONTH15 = "1150"
MONTH18 = "1180"
MONTH21 = "1210"
MONTH24 = "1240"
MONTH27 = "1270"
MONTH3 = "1030"
MONTH30 = "1300"
MONTH33 = "1330"
MONTH36 = "1360"
MONTH6 = "1060"
MONTH9 = "1090"
POSTNATAL = "3000"
SCHEDULE = "schedule"
SCHEDULE_POSTNATAL = "schedule_postnatal"
SCHEDULE_PREGNANCY = "schedule_pregnancy"
VISIT_SCHEDULE = "visit_schedule"
WEEK2 = "1005"



from copy import copy

from django.contrib import admin
from edc_action_item import action_fields, action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from meta_subject.models import UrinePregnancy

from ..admin_site import meta_prn_admin
from ..forms import PregnancyNotificationForm
from ..models import PregnancyNotification


@admin.register(PregnancyNotification, site=meta_prn_admin)
class PregnancyNotificationAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = PregnancyNotificationForm

    additional_instructions = (
        "Important: A positive Urine βhCG must be entered before this form "
        f"may be completed (See `{UrinePregnancy._meta.verbose_name}`)."
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Pregnancy information",
            {"fields": ("bhcg_confirmed", "unconfirmed_details", "edd")},
        ),
        (
            "Followup",
            {"fields": ("may_contact",)},
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "edd",
        "may_contact",
    )

    list_filter = ("edd", "may_contact")

    radio_fields = {
        "bhcg_confirmed": admin.VERTICAL,
        "may_contact": admin.VERTICAL,
    }

    search_fields = ("subject_identifier", "action_identifier", "tracking_identifier")

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        action_flds.remove("action_identifier")
        fields = list(action_flds) + list(fields)
        return fields



from multisite import SiteID

from .defaults import *  # noqa

print(f"Settings file {__file__}")  # noqa

SITE_ID = SiteID(default=1)
EDC_SITES_UAT_DOMAIN = False
ALLOWED_HOSTS = [
    "mnazi-moja.tz.meta3.clinicedc.org",
    "mbagala.tz.meta3.clinicedc.org",
    "mwananyamala.tz.meta3.clinicedc.org",
    "hindu-mandal.tz.meta3.clinicedc.org",
    "temeke.tz.meta3.clinicedc.org",
    "amana.tz.meta3.clinicedc.org",
    "mwananyamala.tz.meta.clinicedc.org",
    "hindu-mandal.tz.meta.clinicedc.org",
    "temeke.tz.meta.clinicedc.org",
    "amana.tz.meta.clinicedc.org",
    "localhost",
]



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_constants.constants import NO, YES
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixin import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from meta_subject.models import UrinePregnancy

from ..models import PregnancyNotification


class PregnancyNotificationFormValidator(FormValidator):
    def clean(self):
        self.required_if(
            NO, field="bhcg_confirmed", field_required="unconfirmed_details"
        )
        if self.instance.id is None and self.cleaned_data.get("bhcg_confirmed") == YES:
            if not UrinePregnancy.objects.filter(
                subject_visit__subject_identifier=self.cleaned_data.get(
                    "subject_identifier"
                ),
                notified=False,
                assay_date__lte=self.cleaned_data.get("report_datetime"),
            ).exists():
                raise forms.ValidationError(
                    "Invalid. A positive Urine βhCG cannot be found. "
                    f"See {UrinePregnancy._meta.verbose_name}"
                )


class PregnancyNotificationForm(
    SiteModelFormMixin, FormValidatorMixin, ActionItemFormMixin, forms.ModelForm
):

    form_validator_cls = PregnancyNotificationFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = PregnancyNotification
        fields = "__all__"



# Generated by Django 3.2.7 on 2021-09-29 20:24

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0090_auto_20210924_0424"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsglu",
            name="glucose_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsglu",
            name="glucose_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="glucose_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="glucose_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
    ]



from copy import copy

from django.contrib import admin
from edc_action_item import action_fields, action_fieldset_tuple
from edc_model_admin import audit_fieldset_tuple

from meta_ae.models import DeathReport

from ...models import LossToFollowup


class EndOfStudyAdminMixin:

    form = None

    additional_instructions = (
        "Note: if the patient is <i>deceased</i>, complete form "
        f"`{DeathReport._meta.verbose_name}` before completing this form. "
        "<BR>If the patient is </i>lost to follow up</i>, complete form "
        f"`{LossToFollowup._meta.verbose_name}` before completing this form."
    )

    fieldsets = (
        [
            "Part 1:",
            {
                "fields": (
                    "subject_identifier",
                    "offschedule_datetime",
                    "offschedule_reason",
                    "other_offschedule_reason",
                    "ltfu_date",
                    "death_date",
                    "comment",
                )
            },
        ],
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "offschedule_datetime",
        "tracking_identifier",
        "action_identifier",
    )

    list_filter = ("offschedule_datetime",)

    radio_fields = {"offschedule_reason": admin.VERTICAL}

    search_fields = ("subject_identifier", "action_identifier", "tracking_identifier")

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        fields = list(action_flds) + list(fields)
        return fields



# Generated by Django 3.2.11 on 2022-03-12 16:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0038_auto_20220312_1929'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalicpreferral',
            name='fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='converted_fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='fbg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='converted_fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='fbg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='converted_fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='fbg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='converted_fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='fbg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='icpreferral',
            name='fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT FBG level'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='converted_fbg_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='fbg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
    ]



from dateutil.relativedelta import relativedelta
from edc_visit_schedule import Schedule, Visit

from ...constants import POSTNATAL, SCHEDULE_POSTNATAL
from .crfs import crfs_36m, crfs_missed
from .crfs import crfs_prn
from .crfs import crfs_prn as default_crfs_prn

schedule = Schedule(
    name=SCHEDULE_POSTNATAL,
    verbose_name="Postnatal 36m Follow-up",
    onschedule_model="meta_prn.onschedulepostnatal",
    offschedule_model="meta_prn.offschedulepostnatal",
    consent_model="meta_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
)

# TODO: appointment date will be 36m from consent

visit = Visit(
    code=POSTNATAL,
    title="Postnatal 36m Followup",
    timepoint=300,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(months=1),
    crfs=crfs_36m,
    crfs_prn=crfs_prn or default_crfs_prn,
    crfs_missed=crfs_missed,
    facility_name="7-day-clinic",
)


schedule.add_visit(visit=visit)



from django.contrib import admin
from django.urls.conf import include, path

from meta_prn.admin_site import meta_prn_admin

urlpatterns = [
    path("meta_prn/", include("meta_prn.urls")),
    path("admin/", meta_prn_admin.urls),
    path("admin/", admin.site.urls),
]



from django.utils.safestring import mark_safe

from ..forms import (
    get_part_one_fields,
    get_part_three_fields,
    get_part_three_vitals_fields,
    get_part_two_fields,
    part_three_comment_fields,
    part_three_glucose_fields,
    part_three_other_fields,
    part_three_pregnancy_fields,
)
from ..forms.field_lists import (
    part_three_creatinine_fields,
    part_three_hba1c_fields,
    part_three_repeat_fbg_fields,
    part_three_repeat_ogtt_fields,
)


def get_part_one_fieldset(collapse=None):

    dct = {
        "description": mark_safe(
            "To be completed by the <u>study clinician</u> or the "
            "<u>research nurse</u> in consultation with the study clinician"
        ),
        "fields": get_part_one_fields(),
    }
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 1", dct


def get_part_two_fieldset(collapse=None):
    dct = {
        "description": mark_safe(
            "To be completed by the <u>study clinician</u> or the "
            "<u>research nurse</u> in consultation with the study clinician"
        ),
        "fields": get_part_two_fields(),
    }
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 2", dct


def get_part_three_fieldset(collapse=None):
    dct = {
        "description": mark_safe("To be completed by the <u>study clinician</u>"),
        "fields": get_part_three_fields(),
    }
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3: Biomedical Indicators", dct


def get_part_three_glucose_fieldset(collapse=None):
    dct = {"fields": part_three_glucose_fields}
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3c:  Glucose Measurements (FBG / OGTT)", dct


def get_part_three_repeat_glucose_fieldset(collapse=None):
    fields = [
        "repeat_glucose_performed",
        "repeat_fasting",
        "repeat_fasting_duration_str",
    ]
    fields.extend(part_three_repeat_fbg_fields)
    fields.extend(part_three_repeat_ogtt_fields)
    dct = {
        "fields": fields,
        "description": (
            "IMPORTANT: If you decide to repeat the FBG / OGTT, do so at least three days "
            "after the first FBG / OGTT"
        ),
    }
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3d: Repeat Glucose Measurements (FBG / OGTT)", dct


def get_part_three_other_fieldset(collapse=None):
    dct = {"fields": part_three_other_fields}
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3e: Creatinine / HbA1c", dct


def get_part_three_creatinine_fieldset(collapse=None):
    dct = {"fields": part_three_creatinine_fields}
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3e: Creatinine", dct


def get_part_three_hba1c_fieldset(collapse=None):
    dct = {"fields": part_three_hba1c_fields}
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3f: HbA1c", dct


def get_part_three_report_datetime_fieldset():
    dct = {"fields": ["part_three_report_datetime"]}
    return "Part 3", dct


def get_part_three_vitals_fieldset(collapse=None):
    dct = {"fields": get_part_three_vitals_fields()}
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3a: Vitals", dct


def get_part_three_pregnancy_fieldset(collapse=None):
    dct = {"fields": part_three_pregnancy_fields}
    if collapse:
        dct.update(classes=("collapse",))
    return "Part 3b: Pregnancy", dct


comments_fieldset = (
    "Additional Comments",
    {
        "fields": (*part_three_comment_fields,),
    },
)

calculated_values_fieldset = (
    "Calculated values",
    {
        "classes": ("collapse",),
        "fields": (
            "sys_blood_pressure_avg",
            "dia_blood_pressure_avg",
            "converted_fbg_value",
            "converted_ogtt_value",
            "converted_fbg2_value",
            "converted_ogtt2_value",
            "converted_creatinine_value",
            "calculated_egfr_value",
        ),
    },
)



from django.db import models
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_crypto_fields.fields import EncryptedCharField
from edc_constants.choices import SELECTION_METHOD, YES_NO, YES_NO_NA, YESDEFAULT_NO
from edc_constants.constants import YES

from ..choices import ETHNICITY
from ..constants import PREG_YES_NO_NA


class PartOneFieldsModelMixin(models.Model):

    screening_consent = models.CharField(
        verbose_name=format_html(
            "Has the subject given his/her verbal consent "
            "to be screened for the <u>META Phase 3</u> trial?"
        ),
        max_length=15,
        choices=YES_NO,
    )

    selection_method = models.CharField(
        verbose_name="How was the patient selected from the outpatients CTC?",
        max_length=25,
        choices=SELECTION_METHOD,
    )

    meta_phase_two = models.CharField(
        verbose_name=format_html(
            "Was the subject enrolled in the <u>META Phase 2</u> trial?"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    hospital_identifier = EncryptedCharField(unique=True, blank=False)

    ethnicity = models.CharField(
        max_length=15, choices=ETHNICITY, help_text="Used for eGFR calculation"
    )

    hiv_pos = models.CharField(
        verbose_name="Is the patient HIV positive", max_length=15, choices=YES_NO
    )

    art_six_months = models.CharField(
        verbose_name=format_html(
            "Has the patient been on anti-retroviral therapy for <u>at least 6 months</u>"
        ),
        max_length=15,
        choices=YES_NO_NA,
    )

    on_rx_stable = models.CharField(
        verbose_name="Is the patient considered to be stable on treatment ",
        max_length=15,
        choices=YES_NO_NA,
        help_text="in regular attendance for care",
    )

    vl_undetectable = models.CharField(
        verbose_name=format_html(
            "Does the patient have a viral load measure of less than 400 copies per ml "
            "taken <u>within the last 6 months</u>"
        ),
        max_length=15,
        choices=YES_NO_NA,
    )

    lives_nearby = models.CharField(
        verbose_name=(
            "Is the patient living within the catchment population of the facility"
        ),
        max_length=15,
        choices=YES_NO,
    )

    # META PHASE_TWO ONLY
    staying_nearby_6 = models.CharField(
        verbose_name=(
            "Is the patient planning to remain in the catchment area "
            "for at least 6 months"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    # META PHASE_THREE ONLY
    staying_nearby_12 = models.CharField(
        verbose_name=format_html(
            "Is the patient planning to remain in the catchment area "
            "for <u>at least 12 months</u>"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    pregnant = models.CharField(
        verbose_name="Is the patient pregnant?", max_length=15, choices=PREG_YES_NO_NA
    )

    continue_part_two = models.CharField(
        verbose_name=mark_safe(
            "Continue with <U>part two</U> of the screening process?"
        ),
        max_length=15,
        choices=YESDEFAULT_NO,
        default=YES,
        help_text=mark_safe(
            "<B>Important</B>: This response will be be automatically "
            "set to YES if:<BR><BR>"
            "- the participant meets the eligibility criteria for part one, or;<BR><BR>"
            "- the eligibility criteria for part two is already complete.<BR>"
        ),
    )

    class Meta:
        abstract = True



from edc_model import models as edc_models
from edc_pharmacy.models import StudyMedicationCrfModelMixin

from ..model_mixins import CrfModelMixin


class StudyMedication(
    StudyMedicationCrfModelMixin, CrfModelMixin, edc_models.BaseUuidModel
):
    class Meta(
        StudyMedicationCrfModelMixin.Meta,
        CrfModelMixin.Meta,
        edc_models.BaseUuidModel.Meta,
    ):
        pass



from django.contrib import admin
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from edc_consent.modeladmin_mixins import ModelAdminConsentMixin
from edc_identifier import SubjectIdentifierError, is_subject_identifier_or_raise
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from meta_screening.models.subject_screening import SubjectScreening
from meta_subject.models import SubjectVisit

from ..admin_site import meta_consent_admin
from ..forms import SubjectConsentForm
from ..models import SubjectConsent


@admin.register(SubjectConsent, site=meta_consent_admin)
class SubjectConsentAdmin(
    ModelAdminConsentMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = SubjectConsentForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "screening_identifier",
                    "subject_identifier",
                    "first_name",
                    "last_name",
                    "initials",
                    "gender",
                    "language",
                    "is_literate",
                    "witness_name",
                    "consent_datetime",
                    "dob",
                    "guardian_name",
                    "is_dob_estimated",
                    "identity",
                    "identity_type",
                    "confirm_identity",
                    "is_incarcerated",
                )
            },
        ),
        (
            "Review Questions",
            {
                "fields": (
                    "consent_reviewed",
                    "study_questions",
                    "assessment_score",
                    "consent_signature",
                    "consent_copy",
                ),
                "description": "The following questions are directed to the interviewer.",
            },
        ),
        audit_fieldset_tuple,
    )

    search_fields = ("subject_identifier", "screening_identifier", "identity")

    radio_fields = {
        "gender": admin.VERTICAL,
        "assessment_score": admin.VERTICAL,
        "consent_copy": admin.VERTICAL,
        "consent_reviewed": admin.VERTICAL,
        "consent_signature": admin.VERTICAL,
        "is_dob_estimated": admin.VERTICAL,
        "identity_type": admin.VERTICAL,
        "is_incarcerated": admin.VERTICAL,
        "is_literate": admin.VERTICAL,
        "language": admin.VERTICAL,
        "study_questions": admin.VERTICAL,
    }

    def delete_view(self, request, object_id, extra_context=None):
        """Prevent deletion if SubjectVisit objects exist."""
        extra_context = extra_context or {}
        obj = SubjectConsent.objects.get(id=object_id)
        try:
            protected = [
                SubjectVisit.objects.get(subject_identifier=obj.subject_identifier)
            ]
        except ObjectDoesNotExist:
            protected = None
        except MultipleObjectsReturned:
            protected = SubjectVisit.objects.filter(
                subject_identifier=obj.subject_identifier
            )
        extra_context.update({"protected": protected})
        return super().delete_view(request, object_id, extra_context)

    def get_next_options(self, request=None, **kwargs):
        """Returns the key/value pairs from the "next" querystring
        as a dictionary.
        """
        next_options = super().get_next_options(request=request, **kwargs)
        try:
            is_subject_identifier_or_raise(next_options["subject_identifier"])
        except SubjectIdentifierError:
            next_options["subject_identifier"] = SubjectScreening.objects.get(
                subject_identifier_as_pk=next_options["subject_identifier"]
            ).subject_identifier
        except KeyError:
            pass
        return next_options



from .birth_outcomes_form import BirthOutcomesForm
from .blood_results import (
    BloodResultsFbcForm,
    BloodResultsGluForm,
    BloodResultsHba1cForm,
    BloodResultsInsForm,
    BloodResultsLftForm,
    BloodResultsLipidForm,
    BloodResultsRftForm,
)
from .complications_glycemia_form import ComplicationsGlycemiaForm
from .concomitant_medication_form import ConcomitantMedicationForm
from .delivery_form import DeliveryForm
from .eq53d3l_form import Eq5d3lForm
from .followup_examination_form import FollowupExaminationForm
from .followup_vitals_form import FollowupVitalsForm
from .glucose_form import GlucoseForm
from .health_economics_form import HealthEconomicsForm
from .health_economics_simple_form import HealthEconomicsSimpleForm
from .hepatitis_test_form import HepatitisTestForm
from .malaria_test_form import MalariaTestForm
from .medication_adherence_form import MedicationAdherenceForm
from .mnsi_form import MnsiForm
from .other_arv_regimens_detail_form import OtherArvRegimensDetailForm
from .other_arv_regimens_form import OtherArvRegimensForm
from .patient_history_form import PatientHistoryForm
from .physical_exam_form import PhysicalExamForm
from .pregnancy_update_form import PregnancyUpdateForm
from .sf12_form import Sf12Form
from .study_medication_form import StudyMedicationForm
from .subject_requisition_form import SubjectRequisitionForm
from .subject_visit_form import SubjectVisitForm
from .subject_visit_missed_form import SubjectVisitMissedForm
from .urine_dipstick_test_form import UrineDipstickTestForm
from .urine_pregnancy_form import UrinePregnancyForm



from django.contrib import admin
from edc_blood_results.admin import BloodResultsModelAdminMixin
from edc_blood_results.fieldsets import BloodResultFieldset

from ...admin_site import meta_subject_admin
from ...forms import BloodResultsInsForm
from ...models import BloodResultsIns
from ..modeladmin import CrfModelAdmin


@admin.register(BloodResultsIns, site=meta_subject_admin)
class BloodResultsInsAdmin(BloodResultsModelAdminMixin, CrfModelAdmin):
    form = BloodResultsInsForm
    fieldsets = BloodResultFieldset(
        BloodResultsIns.lab_panel, model_cls=BloodResultsIns
    ).fieldsets



from edc_identifier.simple_identifier import (
    SimpleTimestampIdentifier,
    SimpleUniqueIdentifier,
)


class AeIdentifier(SimpleUniqueIdentifier):
    random_string_length = 2
    identifier_type = "tracking_identifier"
    template = "AE{device_id}{timestamp}{random_string}"
    identifier_cls = SimpleTimestampIdentifier


class PdIdentifier(SimpleUniqueIdentifier):
    random_string_length = 2
    identifier_type = "pd_identifier"
    template = "PD{device_id}{timestamp}{random_string}"
    identifier_cls = SimpleTimestampIdentifier



from django import forms
from django.apps import apps as django_apps
from edc_adverse_event.forms import DeathReportModelFormMixin
from edc_constants.constants import OTHER
from edc_form_validators import FormValidator

from ..constants import HOSPITAL_CLINIC
from ..models import DeathReport


class DeathReportFormValidator(FormValidator):
    @property
    def cause_of_death_model_cls(self):
        return django_apps.get_model("edc_adverse_event.causeofdeath")

    def clean(self):

        self.required_if(
            HOSPITAL_CLINIC,
            field="death_location",
            field_required="hospital_name",
        )

        self.validate_other_specify(
            field="informant_relationship",
            other_specify_field="other_informant_relationship",
        )

        other = self.cause_of_death_model_cls.objects.get(name=OTHER)
        self.validate_other_specify(
            field="cause_of_death",
            other_specify_field="cause_of_death_other",
            other_stored_value=other.name,
        )

        self.validate_other_specify(
            field="secondary_cause_of_death",
            other_specify_field="secondary_cause_of_death_other",
            other_stored_value=other.name,
        )


class DeathReportForm(DeathReportModelFormMixin, forms.ModelForm):

    subject_identifier = forms.CharField(
        label="Subject identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = DeathReport
        fields = "__all__"



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_blood_results.form_validator_mixins import BloodResultsFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import FormValidator
from edc_lab_panel.panels import fbc_panel

from ...models import BloodResultsFbc


class BloodResultsFbcFormValidator(BloodResultsFormValidatorMixin, FormValidator):
    panel = fbc_panel


class BloodResultsFbcForm(ActionItemFormMixin, CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = BloodResultsFbcFormValidator

    class Meta:
        model = BloodResultsFbc
        fields = "__all__"



from django.db import models
from edc_constants.choices import MARITAL_STATUS
from edc_he.model_mixins import HealthEconomicsEducationModelMixin
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class HealthEconomicsSimple(
    HealthEconomicsEducationModelMixin, CrfModelMixin, edc_models.BaseUuidModel
):

    occupation = models.CharField(
        verbose_name="What is your occupation/profession?", max_length=50
    )

    marital_status = models.CharField(
        verbose_name="Marital status",
        max_length=25,
        choices=MARITAL_STATUS,
    )

    class Meta:
        verbose_name = "Health Economics (M3)"
        verbose_name_plural = "Health Economics (M3)"



# import arrow
# from dateutil import tz
# from django import forms
# from django.apps import apps as django_apps
# from django.conf import settings
# from django.core.exceptions import ObjectDoesNotExist
# from edc_constants.constants import DEAD
# from edc_utils import convert_php_dateformat
#
#
# class ValidateDeathReportMixin:
#
#     death_report_model = "meta_ae.deathreport"
#
#     @property
#     def death_report_model_cls(self):
#         return django_apps.get_model(self.death_report_model)
#
#     def validate_death_report_if_deceased(self):
#         """Validates death report exists of termination_reason
#         is "DEAD.
#
#         Death "date" is the naive date of the settings.TIME_ZONE datetime.
#
#         Note: uses __date field lookup. If using mysql don't forget
#         to load timezone info.
#         """
#         subject_identifier = (
#             self.cleaned_data.get("subject_identifier")
#             or self.instance.subject_identifier
#         )
#
#         try:
#             death_report = self.death_report_model_cls.objects.get(
#                 subject_identifier=subject_identifier
#             )
#         except ObjectDoesNotExist:
#             if self.cleaned_data.get("offschedule_reason") == DEAD:
#                 raise forms.ValidationError(
#                     {
#                         "offschedule_reason": "Patient is deceased, please complete "
#                         "death report form first."
#                     }
#                 )
#         else:
#             local_death_datetime = arrow.get(
#                 death_report.death_datetime, tz.gettz(settings.TIME_ZONE)
#             )
#             if self.cleaned_data.get("death_date") and (
#                 local_death_datetime.date() != self.cleaned_data.get("death_date")
#             ):
#                 expected = local_death_datetime.date().strftime(
#                     convert_php_dateformat(settings.SHORT_DATE_FORMAT)
#                 )
#                 got = self.cleaned_data.get("death_date").strftime(
#                     convert_php_dateformat(settings.SHORT_DATE_FORMAT)
#                 )
#                 raise forms.ValidationError(
#                     {
#                         "death_date": "Date does not match Death Report. "
#                         f"Expected {expected}. Got {got}."
#                     }
#                 )



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from edc_action_item import (
    ModelAdminActionItemMixin,
    action_fields,
    action_fieldset_tuple,
)
from edc_form_label import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, TabularInlineMixin, audit_fieldset_tuple

from ..admin_site import meta_subject_admin
from ..forms import BirthOutcomesForm, DeliveryForm
from ..models import BirthOutcomes, Delivery
from .modeladmin import CrfModelAdminMixin


class BirthOutcomesInlineAdmin(TabularInlineMixin, admin.TabularInline):
    model = BirthOutcomes
    form = BirthOutcomesForm
    extra = 1
    fields = [
        "birth_order",
        "birth_outcome",
        "birth_weight",
    ]


@admin.register(Delivery, site=meta_subject_admin)
class DeliveryAdmin(
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    ModelAdminActionItemMixin,
    SimpleHistoryAdmin,
):

    form = DeliveryForm

    inlines = [BirthOutcomesInlineAdmin]

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Source of information",
            {
                "fields": (
                    "info_available",
                    "info_not_available_reason",
                    "info_source",
                    "info_source_other",
                    "informant_relation",
                    "informant_relation_other",
                )
            },
        ),
        (
            "Delivery",
            {
                "fields": (
                    "delivery_datetime",
                    "delivery_time_estimated",
                    "delivery_location",
                    "delivery_location_other",
                    "delivery_location_name",
                    "delivery_ga",
                    "gm_treated",
                    "maternal_outcome",
                )
            },
        ),
        (
            "Outcomes",
            {"fields": ("fetal_outcome_count",)},
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_visit",
        "birth_outcomes",
        "dashboard",
        "delivery_datetime",
        "maternal_outcome",
        "fetal_outcome_count",
    )

    list_filter = (
        "delivery_datetime",
        "gm_treated",
        "maternal_outcome",
    )

    radio_fields = {
        "info_available": admin.VERTICAL,
        "info_source": admin.VERTICAL,
        "informant_relation": admin.VERTICAL,
        "delivery_time_estimated": admin.VERTICAL,
        "delivery_location": admin.VERTICAL,
        "gm_treated": admin.VERTICAL,
        "maternal_outcome": admin.VERTICAL,
    }

    readonly_fields = action_fields

    search_fields = (
        "subject_visit__subject_identifier",
        "action_identifier",
        "tracking_identifier",
    )

    @admin.display
    def birth_outcomes(self, obj=None, label=None):
        url = reverse("meta_subject_admin:meta_subject_birthoutcomes_changelist")
        url = f"{url}?q={obj.subject_identifier}"
        context = dict(title="Outcomes", url=url, label="Outcomes")
        return render_to_string("dashboard_button.html", context=context)



from django.db import models
from edc_constants.choices import YES_NO
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class DietAndLifestyle(CrfModelMixin, edc_models.BaseUuidModel):
    diet_and_lifestyle = models.CharField(
        verbose_name=(
            "Has the participant received the META3 "
            "approved diet and lifetsyle counseling?"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text="This response is not criteria for eligibility",
    )

    diet_and_lifestyle_duration = models.IntegerField(
        verbose_name=(
            "How much time was spent on the META3 "
            "approved diet and lifetsyle counseling with the patient?"
        ),
        null=True,
        blank=False,
        help_text="Report in minutes. This response is not criteria for eligibility",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Clinic follow up: Examination"
        verbose_name_plural = "Clinic follow up: Examination"



from edc_lab_panel.panels import hba1c_poc_panel
from edc_metadata import NOT_REQUIRED, REQUIRED
from edc_metadata.metadata_rules import (
    CrfRule,
    CrfRuleGroup,
    RequisitionRule,
    RequisitionRuleGroup,
    register,
)

from .predicates import Predicates

pc = Predicates()


@register()
class HealthEconomicsRuleGroup(CrfRuleGroup):

    hecon = CrfRule(
        predicate=pc.health_economics_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["healtheconomicssimple"],
    )

    class Meta:
        app_label = "meta_subject"
        source_model = "meta_subject.subjectvisit"


@register()
class UrinePregnancyRuleGroup(CrfRuleGroup):

    bhcg = CrfRule(
        predicate=pc.urine_pregnancy_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["urinepregnancy"],
    )

    class Meta:
        app_label = "meta_subject"
        source_model = "meta_subject.subjectvisit"


@register()
class HbA1cCrfRuleGroup(CrfRuleGroup):

    hba1c = CrfRule(
        predicate=pc.hba1c_crf_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["bloodresultshba1c"],
    )

    class Meta:
        app_label = "meta_subject"
        source_model = "meta_subject.subjectvisit"


@register()
class HbA1cRequisitionRuleGroup(RequisitionRuleGroup):

    hba1c = RequisitionRule(
        predicate=pc.hba1c_requisition_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_panels=[hba1c_poc_panel],
    )

    class Meta:
        app_label = "meta_subject"
        source_model = "meta_subject.subjectvisit"
        requisition_model = "meta_subject.subjectrequisition"


@register()
class MnsiTestRuleGroup(CrfRuleGroup):

    mnsi = CrfRule(
        predicate=pc.mnsi_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["mnsi"],
    )

    class Meta:
        app_label = "meta_subject"
        source_model = "meta_subject.subjectvisit"



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.safestring import mark_safe
from edc_constants.choices import YES_NO
from edc_model import models as edc_models
from edc_vitals.model_mixins import (
    BloodPressureModelMixin,
    SimpleBloodPressureModelMixin,
)

from ..model_mixins import CrfModelMixin, VitalsFieldsModelMixin


class PhysicalExam(
    SimpleBloodPressureModelMixin,
    BloodPressureModelMixin,
    VitalsFieldsModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):

    irregular_heartbeat = models.CharField(
        verbose_name=mark_safe("Is the heart beat <u>irregular</u>?"),
        max_length=15,
        choices=YES_NO,
    )

    irregular_heartbeat_description = models.TextField(
        "If the heartbeat is <u>irregular</u>, please describe", null=True, blank=True
    )

    waist_circumference = models.DecimalField(
        verbose_name="Waist circumference",
        max_digits=5,
        decimal_places=1,
        validators=[MinValueValidator(50.0), MaxValueValidator(175.0)],
        help_text="in centimeters",
        null=True,
    )

    jaundice = models.CharField(verbose_name="Jaundice?", max_length=15, choices=YES_NO)

    peripheral_oedema = models.CharField(
        verbose_name="Presence of peripheral oedema?", max_length=15, choices=YES_NO
    )

    abdominal_tenderness = models.CharField(
        verbose_name="Abdominal tenderness on palpation?", max_length=15, choices=YES_NO
    )

    abdominal_tenderness_description = models.TextField(
        verbose_name="If YES, abdominal tenderness, please describe",
        null=True,
        blank=True,
    )

    enlarged_liver = models.CharField(
        verbose_name="Enlarged liver on palpation?", max_length=15, choices=YES_NO
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Physical Exam"
        verbose_name_plural = "Physical Exams"



import re

from django.db.models import Q
from edc_constants.constants import ABNORMAL
from edc_dashboard.view_mixins import (
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
)
from edc_dashboard.views import ListboardView as BaseListboardView
from edc_navbar import NavbarViewMixin

from meta_edc.meta_version import get_meta_version

from ...model_wrappers import ScreeningPartOneModelWrapper
from .filters import ListboardViewFilters


class ListboardView(
    EdcViewMixin,
    NavbarViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    BaseListboardView,
):

    listboard_template = "screening_listboard_template"
    listboard_url = "screening_listboard_url"
    listboard_panel_style = "info"
    listboard_fa_icon = "fa-user-plus"
    listboard_view_filters = ListboardViewFilters()
    listboard_model = "meta_screening.screeningpartone"
    listboard_view_permission_codename = "edc_dashboard.view_screening_listboard"

    alternate_search_attr = "screening_identifier"

    model_wrapper_cls = ScreeningPartOneModelWrapper
    navbar_selected_item = "screened_subject"
    ordering = "-report_datetime"
    paginate_by = 10
    search_form_url = "screening_listboard_url"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            subject_screening_add_url=self.get_subject_screening_add_url(),
            ABNORMAL=ABNORMAL,
            meta_version=get_meta_version(),
        )
        return context

    def get_subject_screening_add_url(self):
        return self.listboard_model_cls().get_absolute_url()

    def get_queryset_filter_options(self, request, *args, **kwargs):
        options = super().get_queryset_filter_options(request, *args, **kwargs)
        if kwargs.get("screening_identifier"):
            options.update({"screening_identifier": kwargs.get("screening_identifier")})
        return options

    def extra_search_options(self, search_term):
        q_objects = []
        if re.match(r"^[A-Z\-]+$", search_term):
            q_objects.append(Q(initials__exact=search_term.upper()))
            q_objects.append(
                Q(screening_identifier__icontains=search_term.replace("-", "").upper())
            )
        if re.match(r"^[0-9\-]+$", search_term):
            q_objects.append(Q(hospital_identifier__exact=search_term))
            q_objects.append(Q(subject_identifier__icontains=search_term))
        return q_objects



# Generated by Django 3.2.11 on 2022-03-17 22:33

import _socket
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_identifier.managers
import edc_model.models.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_schedule.model_mixins.schedule_model_mixin
import simple_history.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('edc_action_item', '0028_auto_20210203_0706'),
        ('sites', '0002_alter_domain_unique'),
        ('meta_prn', '0021_auto_20220316_2147'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalOffSchedulePostnatal',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, db_index=True, editable=False, help_text='System auto field. UUID primary key.')),
                ('subject_identifier', models.CharField(db_index=True, max_length=50)),
                ('tracking_identifier', models.CharField(db_index=True, max_length=32)),
                ('action_identifier', models.CharField(db_index=True, max_length=50)),
                ('parent_action_identifier', models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True)),
                ('related_action_identifier', models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True)),
                ('action_item_reason', models.TextField(editable=False, null=True)),
                ('history_id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('report_datetime', models.DateTimeField(editable=False)),
                ('offschedule_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, validators=[edc_protocol.validators.datetime_not_before_study_start, edc_model.models.validators.date.datetime_not_future], verbose_name='Date and time subject taken off schedule')),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('parent_action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('related_action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('site', models.ForeignKey(blank=True, db_constraint=False, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='sites.site')),
            ],
            options={
                'verbose_name': 'historical off schedule postnatal',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='HistoricalOnSchedulePostnatal',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, db_index=True, editable=False, help_text='System auto field. UUID primary key.')),
                ('subject_identifier', models.CharField(db_index=True, max_length=50)),
                ('report_datetime', models.DateTimeField(editable=False)),
                ('onschedule_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, validators=[edc_protocol.validators.datetime_not_before_study_start, edc_model.models.validators.date.datetime_not_future])),
                ('history_id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('site', models.ForeignKey(blank=True, db_constraint=False, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='sites.site')),
            ],
            options={
                'verbose_name': 'historical on schedule postnatal',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='OffSchedulePostnatal',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, editable=False, help_text='System auto field. UUID primary key.', primary_key=True, serialize=False)),
                ('subject_identifier', models.CharField(max_length=50, unique=True)),
                ('tracking_identifier', models.CharField(max_length=32, unique=True)),
                ('action_identifier', models.CharField(max_length=50, unique=True)),
                ('parent_action_identifier', models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True)),
                ('related_action_identifier', models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True)),
                ('action_item_reason', models.TextField(editable=False, null=True)),
                ('report_datetime', models.DateTimeField(editable=False)),
                ('offschedule_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, validators=[edc_protocol.validators.datetime_not_before_study_start, edc_model.models.validators.date.datetime_not_future], verbose_name='Date and time subject taken off schedule')),
                ('action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem')),
                ('parent_action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem')),
                ('related_action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem')),
                ('site', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='sites.site')),
            ],
            options={
                'abstract': False,
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', edc_visit_schedule.model_mixins.schedule_model_mixin.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='OnSchedulePostnatal',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, editable=False, help_text='System auto field. UUID primary key.', primary_key=True, serialize=False)),
                ('subject_identifier', models.CharField(max_length=50, unique=True)),
                ('report_datetime', models.DateTimeField(editable=False)),
                ('onschedule_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, validators=[edc_protocol.validators.datetime_not_before_study_start, edc_model.models.validators.date.datetime_not_future])),
                ('site', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='sites.site')),
            ],
            options={
                'abstract': False,
            },
            managers=[
                ('on_site', edc_visit_schedule.model_mixins.schedule_model_mixin.CurrentSiteManager()),
                ('objects', edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='delivery',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='delivery',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='delivery',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='delivery',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='delivery',
            name='site',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='delivery',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='history_user',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='site',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='history_user',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='site',
        ),
        migrations.DeleteModel(
            name='BirthOutcomes',
        ),
        migrations.DeleteModel(
            name='Delivery',
        ),
        migrations.DeleteModel(
            name='HistoricalBirthOutcomes',
        ),
        migrations.DeleteModel(
            name='HistoricalDelivery',
        ),
        migrations.AddIndex(
            model_name='onschedulepostnatal',
            index=models.Index(fields=['id', 'subject_identifier', 'onschedule_datetime', 'site'], name='meta_prn_on_id_e5ef52_idx'),
        ),
        migrations.AddIndex(
            model_name='offschedulepostnatal',
            index=models.Index(fields=['id', 'subject_identifier', 'offschedule_datetime', 'site'], name='meta_prn_of_id_3c0260_idx'),
        ),
    ]



# Generated by Django 3.2.11 on 2022-03-24 00:04

import django.core.validators
from django.db import migrations, models
import django_crypto_fields.fields.encrypted_text_field
import edc_model.models.validators.date
import edc_protocol.validators


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0101_auto_20220316_2147'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='delivery',
            name='informant_contact',
        ),
        migrations.RemoveField(
            model_name='delivery',
            name='informant_is_patient',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='informant_contact',
        ),
        migrations.RemoveField(
            model_name='historicaldelivery',
            name='informant_is_patient',
        ),
        migrations.AddField(
            model_name='delivery',
            name='info_available',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='Yes', help_text='If NO, please explain below', max_length=5, verbose_name='Were you able to obtain a report on the delivery?'),
        ),
        migrations.AddField(
            model_name='delivery',
            name='info_not_available_reason',
            field=models.TextField(null=True, verbose_name='If the report was not available, please explain?'),
        ),
        migrations.AddField(
            model_name='delivery',
            name='info_source',
            field=models.CharField(choices=[('patient', 'Study participant'), ('OTHER', 'Other'), ('N/A', 'Not applicable')], default='patient', max_length=25, verbose_name='Who / what is the MAIN source of this information?'),
        ),
        migrations.AddField(
            model_name='delivery',
            name='info_source_other',
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(blank=True, help_text=' (Encryption: AES local)', max_length=71, null=True, verbose_name='If not reported from study participant, please give the name and contact details of the informant.'),
        ),
        migrations.AddField(
            model_name='historicaldelivery',
            name='info_available',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='Yes', help_text='If NO, please explain below', max_length=5, verbose_name='Were you able to obtain a report on the delivery?'),
        ),
        migrations.AddField(
            model_name='historicaldelivery',
            name='info_not_available_reason',
            field=models.TextField(null=True, verbose_name='If the report was not available, please explain?'),
        ),
        migrations.AddField(
            model_name='historicaldelivery',
            name='info_source',
            field=models.CharField(choices=[('patient', 'Study participant'), ('OTHER', 'Other'), ('N/A', 'Not applicable')], default='patient', max_length=25, verbose_name='Who / what is the MAIN source of this information?'),
        ),
        migrations.AddField(
            model_name='historicaldelivery',
            name='info_source_other',
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(blank=True, help_text=' (Encryption: AES local)', max_length=71, null=True, verbose_name='If not reported from study participant, please give the name and contact details of the informant.'),
        ),
        migrations.AddField(
            model_name='historicalmedicationadherence',
            name='pill_count_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='Yes', max_length=5, verbose_name='Was a pill count performed'),
        ),
        migrations.AddField(
            model_name='medicationadherence',
            name='pill_count_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='Yes', max_length=5, verbose_name='Was a pill count performed'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='delivery_datetime',
            field=models.DateTimeField(help_text='If TIME unknown, estimate', null=True, validators=[edc_model.models.validators.date.datetime_not_future, edc_protocol.validators.datetime_not_before_study_start], verbose_name='Date and time of delivery :'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='delivery_ga',
            field=models.IntegerField(null=True, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(45)], verbose_name='Gestational age at delivery'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='delivery_location',
            field=models.CharField(choices=[('home', 'At home'), ('hospital_clinic', 'Hospital / Clinic'), ('OTHER', 'Other location, specify'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='Where did the delivery occur?'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='delivery_time_estimated',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], max_length=3, verbose_name='Is the delivery TIME estimated?'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='fetal_outcome_count',
            field=models.IntegerField(help_text='Each to be reported individually below', null=True, verbose_name='Number of births / fetal / neonatal outcomes'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='gm_treated',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=5, verbose_name='Was the participant treated for gestational diabetes?'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='informant_relation',
            field=models.CharField(choices=[('husband_wife', 'Husband/wife'), ('Parent', 'Parent'), ('child', 'Child'), ('unknown', 'Unknown'), ('OTHER', 'Other'), ('hospital_clinic', 'Hospital / Clinic records'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='Informants relationship to the participant?'),
        ),
        migrations.AlterField(
            model_name='delivery',
            name='maternal_outcome',
            field=models.CharField(choices=[('no_complications', 'No complications'), ('complications_full_recovery', 'Complications with full recovery'), ('complications_ongoing_recovery', 'Complications with ongoing recovery'), ('dead', 'Maternal mortality'), ('N/A', 'Not applicable')], default='N/A', max_length=50, verbose_name='What was the maternal outcome of the pregnancy?'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='delivery_datetime',
            field=models.DateTimeField(help_text='If TIME unknown, estimate', null=True, validators=[edc_model.models.validators.date.datetime_not_future, edc_protocol.validators.datetime_not_before_study_start], verbose_name='Date and time of delivery :'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='delivery_ga',
            field=models.IntegerField(null=True, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(45)], verbose_name='Gestational age at delivery'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='delivery_location',
            field=models.CharField(choices=[('home', 'At home'), ('hospital_clinic', 'Hospital / Clinic'), ('OTHER', 'Other location, specify'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='Where did the delivery occur?'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='delivery_time_estimated',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], max_length=3, verbose_name='Is the delivery TIME estimated?'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='fetal_outcome_count',
            field=models.IntegerField(help_text='Each to be reported individually below', null=True, verbose_name='Number of births / fetal / neonatal outcomes'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='gm_treated',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=5, verbose_name='Was the participant treated for gestational diabetes?'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='informant_relation',
            field=models.CharField(choices=[('husband_wife', 'Husband/wife'), ('Parent', 'Parent'), ('child', 'Child'), ('unknown', 'Unknown'), ('OTHER', 'Other'), ('hospital_clinic', 'Hospital / Clinic records'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='Informants relationship to the participant?'),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='maternal_outcome',
            field=models.CharField(choices=[('no_complications', 'No complications'), ('complications_full_recovery', 'Complications with full recovery'), ('complications_ongoing_recovery', 'Complications with ongoing recovery'), ('dead', 'Maternal mortality'), ('N/A', 'Not applicable')], default='N/A', max_length=50, verbose_name='What was the maternal outcome of the pregnancy?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectvisit',
            name='last_alive_date',
            field=models.DateField(blank=True, help_text='Will be validated on the Death report', null=True, validators=[edc_protocol.validators.date_not_before_study_start, edc_model.models.validators.date.date_not_future], verbose_name='Date participant last known alive'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectvisit',
            name='survival_status',
            field=models.CharField(choices=[('alive', 'Alive'), ('dead', 'Deceased'), ('unknown', 'Unknown')], default='alive', help_text='If YES, submit Death report', max_length=10, null=True, verbose_name="Participant's survival status"),
        ),
        migrations.AlterField(
            model_name='subjectvisit',
            name='last_alive_date',
            field=models.DateField(blank=True, help_text='Will be validated on the Death report', null=True, validators=[edc_protocol.validators.date_not_before_study_start, edc_model.models.validators.date.date_not_future], verbose_name='Date participant last known alive'),
        ),
        migrations.AlterField(
            model_name='subjectvisit',
            name='survival_status',
            field=models.CharField(choices=[('alive', 'Alive'), ('dead', 'Deceased'), ('unknown', 'Unknown')], default='alive', help_text='If YES, submit Death report', max_length=10, null=True, verbose_name="Participant's survival status"),
        ),
    ]



from edc_adverse_event.constants import DEATH_REPORT_TMG_SECOND_ACTION
from edc_adverse_event.model_mixins import (
    DeathReportTmgSecondManager,
    DeathReportTmgSecondSiteManager,
)

from .death_report_tmg import DeathReportTmg


class DeathReportTmgSecond(DeathReportTmg):

    action_name = DEATH_REPORT_TMG_SECOND_ACTION

    on_site = DeathReportTmgSecondSiteManager()

    objects = DeathReportTmgSecondManager()

    class Meta:
        proxy = True
        verbose_name = "Death Report TMG (2nd)"
        verbose_name_plural = "Death Report TMG (2nd)"



ALL_OF_THE_TIME = "all_of_the_time"
APPT = "appt"
APPT_OTHER = "other_routine_appt"
DELIVERY_ACTION = "delivery_action"
FOLLOWUP_EXAMINATION_ACTION = "followup-examination-ae"
GOOD_BIT_OF_THE_TIME = "good_bit_of_the_time"
LITTLE_OF_THE_TIME = "little_of_the_time"
LIVE_AT_TERM = "live_at_term"
LIVE_PRETERM = "live_preterm"
LOW_EGFR_ACTION = "low-egfr"
MISSED_VISIT_ACTION = "missed_visit_action"
MOST_OF_THE_TIME = "most_of_the_time"
NONE_OF_THE_TIME = "none_of_the_time"
NO_COMPLICATIONS = "no_complications"
SOME_OF_THE_TIME = "some_of_the_time"
URINE_PREGNANCY_ACTION = "urine_pregnancy_action"



from edc_constants.constants import QUESTION_RETIRED
from edc_model.models import BaseUuidModel
from edc_screening.model_mixins import ScreeningModelMixin
from edc_screening.screening_identifier import (
    ScreeningIdentifier as BaseScreeningIdentifier,
)

from .calculated_model_mixin import CalculatedModelMixin
from .eligibility_model_mixin import EligibilityModelMixin
from .part_four_field_model_mixin import PartFourFieldsModelMixin
from .part_one_fields_model_mixin import PartOneFieldsModelMixin
from .part_three_fields_model_mixin import PartThreeFieldsModelMixin
from .part_two_fields_model_mixin import PartTwoFieldsModelMixin


class SubjectScreeningModelError(Exception):
    pass


class ScreeningIdentifier(BaseScreeningIdentifier):

    template = "S{random_string}"


class SubjectScreening(
    PartOneFieldsModelMixin,
    PartTwoFieldsModelMixin,
    PartThreeFieldsModelMixin,
    PartFourFieldsModelMixin,
    EligibilityModelMixin,
    CalculatedModelMixin,
    ScreeningModelMixin,
    BaseUuidModel,
):

    identifier_cls = ScreeningIdentifier

    def save(self, *args, **kwargs):
        if self._meta.label_lower == "meta_screening.subjectscreening":
            raise SubjectScreeningModelError(
                "Unable to save. Save via P1-3 proxy models."
            )
        self.consent_ability = QUESTION_RETIRED
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "Subject Screening"
        verbose_name_plural = "Subject Screening"



from django import forms
from django.urls.base import reverse
from django.utils.safestring import mark_safe
from edc_dashboard.url_names import url_names
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin

from ..form_validators import SubjectRefusalFormValidator
from ..models import SubjectRefusal, SubjectScreening


class ScreeningFormMixin:
    def clean(self):
        cleaned_data = super().clean()
        screening_identifier = cleaned_data.get("screening_identifier")
        if screening_identifier:
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=screening_identifier
            )
            if not subject_screening.eligible:
                url_name = url_names.get("screening_listboard_url")
                url = reverse(
                    url_name,
                    kwargs={"screening_identifier": self.instance.screening_identifier},
                )
                msg = mark_safe(
                    "Not allowed. Subject is not eligible. "
                    f'See subject <A href="{url}?q={screening_identifier}">'
                    f"{screening_identifier}</A>"
                )
                raise forms.ValidationError(msg)
        return cleaned_data


class SubjectRefusalForm(
    AlreadyConsentedFormMixin, ScreeningFormMixin, FormValidatorMixin, forms.ModelForm
):

    form_validator_cls = SubjectRefusalFormValidator

    def clean(self):
        cleaned_data = super().clean()
        return cleaned_data

    class Meta:
        model = SubjectRefusal
        fields = "__all__"



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_tracking.form_validators import VisitFormValidator

from ..models import SubjectVisit


class SubjectVisitFormValidator(VisitFormValidator):
    validate_missed_visit_reason = False


class SubjectVisitForm(SiteModelFormMixin, FormValidatorMixin, forms.ModelForm):

    form_validator_cls = SubjectVisitFormValidator

    class Meta:
        model = SubjectVisit
        fields = "__all__"



from copy import deepcopy

from django.test import TestCase
from edc_appointment.constants import IN_PROGRESS_APPT
from edc_appointment.models import Appointment
from edc_constants.constants import COMPLETE, NO, YES
from edc_visit_tracking.constants import SCHEDULED

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_subject.forms import PhysicalExamForm
from meta_subject.models import SubjectVisit
from meta_visit_schedule.constants import DAY1


class TestPhysicalExam(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.data = {
            "abdominal_tenderness": NO,
            "dia_blood_pressure": 100,
            "abdominal_tenderness_description": None,
            "enlarged_liver": YES,
            "heart_rate": 40,
            "irregular_heartbeat": NO,
            "irregular_heartbeat_description": None,
            "jaundice": YES,
            "oxygen_saturation": 10,
            "peripheral_oedema": NO,
            "report_datetime": NO,
            "respiratory_rate": 30,
            "sys_blood_pressure": 80,
            "temperature": 37,
            "waist_circumference": 100,
            "weight": 65,
            "crf_status": COMPLETE,
        }

        self.subject_visit = self.get_subject_visit()

        self.data.update(
            subject_visit=self.subject_visit.pk,
            report_datetime=self.subject_visit.report_datetime,
        )

    def start_visit(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        subject_identifier = subject_consent.subject_identifier

        appointment = Appointment.objects.get(
            subject_identifier=subject_identifier, visit_code=DAY1
        )
        appointment.appt_status = IN_PROGRESS_APPT
        appointment.save()
        return SubjectVisit.objects.create(appointment=appointment, reason=SCHEDULED)

    def test_ok(self):
        form = PhysicalExamForm(data=self.data)
        form.is_valid()
        self.assertEqual(form._errors, {})

    def test_irregular_heartbeat(self):
        data = deepcopy(self.data)
        data.update(
            irregular_heartbeat=YES, irregular_heartbeat_description="blah blah blah"
        )
        form = PhysicalExamForm(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})

        data.update(
            irregular_heartbeat=NO, irregular_heartbeat_description="blah blah blah"
        )
        form = PhysicalExamForm(data=data)
        form.is_valid()
        self.assertEqual(
            form._errors.get("irregular_heartbeat_description"),
            ["This field is not required."],
        )

    def test_abdominal_tenderness(self):
        data = deepcopy(self.data)
        data.update(abdominal_tenderness=NO, abdominal_tenderness_description=None)
        form = PhysicalExamForm(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})

        data.update(
            abdominal_tenderness=YES, abdominal_tenderness_description="blah blah blah"
        )
        form = PhysicalExamForm(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})

        data.update(
            abdominal_tenderness=NO, abdominal_tenderness_description="blah blah blah"
        )
        form = PhysicalExamForm(data=data)
        form.is_valid()
        self.assertEqual(
            form._errors.get("abdominal_tenderness_description"),
            ["This field is not required."],
        )



from django.contrib.auth import admin
from django.urls.conf import include, path
from django.views.generic.base import RedirectView

from meta_ae.admin_site import meta_ae_admin

urlpatterns = [
    path("admin/", meta_ae_admin.urls),
    path("admin/", admin.site.urls),
    path("accounts/", include("edc_auth.urls")),
    path("edc_adverse_event/", include("edc_adverse_event.urls")),
    path("edc_auth/", include("edc_auth.urls")),
    path("edc_consent/", include("edc_consent.urls")),
    path("edc_dashboard/", include("edc_dashboard.urls")),
    path("edc_device/", include("edc_device.urls")),
    path("edc_export/", include("edc_export.urls")),
    path("edc_protocol/", include("edc_protocol.urls")),
    path("edc_randomization/", include("edc_randomization.urls")),
    path("edc_reference/", include("edc_reference.urls")),
    path("edc_visit_schedule/", include("edc_visit_schedule.urls")),
    path("ae/", include("meta_ae.urls")),
    path("meta_consent/", include("meta_consent.urls")),
    path(
        "administration", RedirectView.as_view(url="admin/"), name="administration_url"
    ),
    path("", RedirectView.as_view(url="admin/"), name="home_url"),
]



from django.apps import AppConfig as DjangoAppConfig

from meta_edc.meta_version import get_meta_version


class AppConfig(DjangoAppConfig):
    name = "meta_reference"
    verbose_name = f"META{get_meta_version()}: Reference"



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist


class MetaCrfReportMixin:
    weight_model = "meta_subject.followup"

    @property
    def unblinded(self):
        UnblindingRequest = django_apps.get_model("meta_prn.unblindingrequest")
        try:
            unblinded = UnblindingRequest.objects.get(
                subject_identifier=self.subject_identifier, approved=True
            )
        except ObjectDoesNotExist:
            unblinded = False
        return unblinded



# Generated by Django 3.2.11 on 2022-03-09 19:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0018_auto_20220309_2106'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='birthoutcomes',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'export', 'import'), 'get_latest_by': 'modified', 'ordering': ['subject_identifier', 'birth_order'], 'verbose_name': 'Birth Outcomes', 'verbose_name_plural': 'Birth Outcomes'},
        ),
        migrations.RenameField(
            model_name='birthoutcomes',
            old_name='maternal_identifier',
            new_name='subject_identifier',
        ),
        migrations.RenameField(
            model_name='historicalbirthoutcomes',
            old_name='maternal_identifier',
            new_name='subject_identifier',
        ),
        migrations.AlterUniqueTogether(
            name='birthoutcomes',
            unique_together={('subject_identifier', 'birth_order')},
        ),
    ]



from edc_constants.constants import NOT_APPLICABLE, OTHER, PATIENT, UNKNOWN

from .constants import CLINICIAN, INVESTIGATOR, PREGNANCY, SAE

CLINICAL_WITHDRAWAL_REASONS = (
    ("pregnancy", "Pregnancy"),
    ("kidney_disease", "Development of chronic kidney disease"),
    ("liver_disease", "Development of chronic liver disease"),
    ("intercurrent_illness", "Intercurrent illness which prevents further treatment"),
    ("investigator_decision", "Investigator decision"),
    (
        OTHER,
        (
            "Other condition that justifies the discontinuation of "
            "treatment in the clinician’s opinion (specify below)"
        ),
    ),
)

# PHASE_THREE ONLY

INFORMANT_RELATIONSHIP = (
    ("husband_wife", "Husband/wife"),
    ("Parent", "Parent"),
    ("child", "Child"),
    (UNKNOWN, "Unknown"),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)


TOXICITY_WITHDRAWAL_REASONS = (
    ("lactic_acidosis", "Development of lactic acidosis or hyperlactatemia"),
    ("hepatomegaly", "Development of hepatomegaly with steatosis"),
    (OTHER, "Other (specify below)"),
)

LOSS_CHOICES = (
    ("unknown_address", "Changed to an unknown address"),
    ("never_returned", "Did not return despite reminders"),
    ("bad_contact_details", "Inaccurate contact details"),
    (OTHER, "Other"),
)

PROTOCOL_VIOLATION = (
    ("failure_to_obtain_informed_consent", "Failure to obtain informed " "consent"),
    ("enrollment_of_ineligible_patient", "Enrollment of ineligible patient"),
    (
        "screening_procedure not done",
        "Screening procedure required by " "protocol not done",
    ),
    (
        "screening_or_on-study_procedure",
        "Screening or on-study procedure/lab " "work required not done",
    ),
    (
        "incorrect_research_treatment",
        "Incorrect research treatment given to " "patient",
    ),
    (
        "procedure_not_completed",
        "On-study procedure required by protocol not " "completed",
    ),
    ("visit_non-compliance", "Visit non-compliance"),
    ("medication_stopped_early", "Medication stopped early"),
    ("medication_noncompliance", "Medication_noncompliance"),
    (
        "national_regulations_not_met",
        "Standard WPD, ICH-GCP, local/national " "regulations not met",
    ),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)

ACTION_REQUIRED = (
    ("remain_on_study", "Participant to remain on trial"),
    ("to_be_withdrawn", "Participant to be withdrawn from trial"),
    (
        "remain_on_study_modified",
        "Patient remains on study but data analysis will be modified",
    ),
)


REASON_STUDY_TERMINATED = ()

WITHDRAWAL_STUDY_MEDICATION_REASONS = (
    (PREGNANCY, "Pregnancy"),
    (SAE, "Participant is experiencing a serious adverse event"),
    (
        CLINICIAN,
        "Other condition that justifies discontinuation of "
        "treatment in the clinician’s opinion (specify below)",
    ),
    (INVESTIGATOR, " Investigator decision"),
    (PATIENT, "Patient decision"),
    (OTHER, "Other reason (specify below)"),
)



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from .constants import OFFSCHEDULE_ACTION, OFFSCHEDULE_PREGNANCY_ACTION


class PregnancyMixin:
    """A mixin for off schedule action items"""

    @property
    def offschedule_pregnancy_model_cls(self):
        return django_apps.get_model("meta_prn.offschedulepregnancy")

    def get_next_offschedule_action(self):
        """Returns next off schedule action name"""
        try:
            self.offschedule_pregnancy_model_cls.objects.get(
                subject_identifier=self.subject_identifier
            )
        except ObjectDoesNotExist:
            return OFFSCHEDULE_ACTION
        else:
            return OFFSCHEDULE_PREGNANCY_ACTION



from copy import copy

from django.contrib import admin
from edc_action_item import action_fields, action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_prn_admin
from ..forms import LossToFollowupForm
from ..models import LossToFollowup


@admin.register(LossToFollowup, site=meta_prn_admin)
class LossToFollowupAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = LossToFollowupForm

    # additional_instructions = (
    #     "Note:"
    # )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Loss to followup",
            {
                "fields": (
                    "last_seen_datetime",
                    "number_consecutive_missed_visits",
                    "last_missed_visit_datetime",
                    "home_visited",
                    "home_visit_detail",
                    "loss_category",
                    "loss_category_other",
                    "comment",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "last_seen_datetime",
        "number_consecutive_missed_visits",
        "home_visited",
    )

    list_filter = (
        "last_seen_datetime",
        "last_missed_visit_datetime",
        "number_consecutive_missed_visits",
    )

    radio_fields = {
        "home_visited": admin.VERTICAL,
        "loss_category": admin.VERTICAL,
    }

    search_fields = ("subject_identifier", "action_identifier", "tracking_identifier")

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        action_flds.remove("action_identifier")
        fields = list(action_flds) + list(fields)
        return fields



from django.db import models
from edc_constants.choices import GENDER
from edc_model import models as edc_models
from edc_randomization.models import RandomizationListModelMixin


class RandomizationList(RandomizationListModelMixin, edc_models.BaseUuidModel):

    gender = models.CharField(max_length=25, choices=GENDER)

    class Meta(RandomizationListModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Randomization List (Phase Three)"
        verbose_name_plural = "Randomization List (Phase Three)"



# for reference by old migrations
from edc_lab.managers import RequisitionManager as Manager  # noqa
from edc_lab.model_mixins import RequisitionModelMixin
from edc_model import models as edc_models
from edc_reference.model_mixins import ReferenceModelMixin


class SubjectRequisition(
    RequisitionModelMixin, ReferenceModelMixin, edc_models.BaseUuidModel
):
    class Meta(RequisitionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        pass



# Generated by Django 3.2.11 on 2022-01-28 14:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0029_auto_20211123_1645'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalscreeningpartone',
            name='diet_and_lifestyle',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningpartone',
            name='diet_and_lifestyle_duration',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningpartthree',
            name='diet_and_lifestyle',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningpartthree',
            name='diet_and_lifestyle_duration',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningparttwo',
            name='diet_and_lifestyle',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningparttwo',
            name='diet_and_lifestyle_duration',
        ),
        migrations.RemoveField(
            model_name='historicalsubjectscreening',
            name='diet_and_lifestyle',
        ),
        migrations.RemoveField(
            model_name='historicalsubjectscreening',
            name='diet_and_lifestyle_duration',
        ),
        migrations.RemoveField(
            model_name='subjectscreening',
            name='diet_and_lifestyle',
        ),
        migrations.RemoveField(
            model_name='subjectscreening',
            name='diet_and_lifestyle_duration',
        ),
    ]



from edc_mnsi.model_mixins import MnsiModelMixin
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class Mnsi(
    MnsiModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):
    class Meta(MnsiModelMixin.Meta, CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        pass



# Generated by Django 3.2.4 on 2021-07-15 20:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0063_auto_20210715_0337"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="haemoglobin_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="hct_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="mch_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="mchc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="mcv_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="platelets_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="rbc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="wbc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="glucose_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="hba1c_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="albumin_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alp_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alt_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="amylase_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="ast_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="ggt_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="chol_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="hdl_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="ldl_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="trig_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="creatinine_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="egfr_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="urea_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="uric_acid_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="haemoglobin_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="hct_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="mch_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="mchc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="mcv_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="platelets_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="rbc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="wbc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="glucose_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="hba1c_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="albumin_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alp_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alt_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="amylase_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="ast_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="ggt_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="chol_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="hdl_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="ldl_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="trig_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="creatinine_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="egfr_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="urea_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="uric_acid_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
    ]
